resource NDeclGerNeu = NDeclGer ** {
  flags
    coding = utf8 ;

  -- Declarations:
  oper 
    declSg', declPl' : Str -> Gender -> (Case => Str) ;

    ePlural, nullPlural, enPlural, nPlural, 
    erPlural, rPlural, sPlural : Str -> (Case => Str) ;

    esSingular, sSingular, enSingular, nSingular, 
    nullSingular : Str -> (Case => Str) ;

    consonant, vowel : pattern Str ; 
  param 
    PlType ; SgType ;

  -- Implementations:
  oper
 
    declSg' str g = table Case [str; str; str; str] ; -- default

    ePlural = \s -> table Case [s+"e";s+"e";s+"en";s+"e"] ;
    nullPlural = \s -> table Case [s; s; s+"n"; s] ;
    enPlural= \s -> table Case [s+"en";s+"en";s+"en";s+"en"] ;
    nPlural = \s -> table Case [s+"n";s+"n";s+"n";s+"n"] ;
    erPlural= \s -> table Case [s+"er";s+"ern";s+"er";s+"er"] ;
    rPlural = \s -> table Case [s+"r";s+"r";s+"r";s+"r"] ;
    sPlural = \s -> table Case [s+"s";s+"s";s+"s";s+"s"] ;

    consonant = #("b" | "c" | "d" | "f" | "g" | "h" | "j" | "k" |
                  "l" | "m" | "n" | "p" | "q" | "r" | "s" | "t" |
                  "v" | "w" | "x" | "y" | "z") ;

    vowel = #("a" | "ee" | "ie" | "o" | "u") ;  -- klingende Vokale

    declPl' nomSg g = case <nomSg,g> of {
         -- regelmäßige Fälle 
         -- 1.Fall: e-Plural
         < x+("bold" | "ig" | "ich" | "ig" | "rich" | 
              "ian" | "jan" | "sal" | "icht" ), 
           _g >              => ePlural nomSg ;
         < x+"nis" , _g >    => ePlural (nomSg+"s") ;
         -- 2.Fall: nullPlural
         < x+("ler" | "ner" | "le" | "en" | "sel" | "tel" | 
              "chen" | "lein" | "erl"), 
           Masc|Neut >       => nullPlural nomSg ;
         < "Muskel" | "Stachel",
           _ >               => nPlural nomSg ;
         < x+("el" | "er"),         -- (bei vielen solchen)
           Masc|Neut >       => nullPlural nomSg ; 
         -- 3.Fall: en/n-Plural
         < x+("rei" | "lei" | "in" | "heit" | "keit" |
              "schaft" | "ung" | "el"),
           Fem >             => enPlural nomSg ;
         < "Mutter" | "Tochter",
           Fem >             => table { c => "TODO" } ; -- Umlaut
         < x+"er", Fem >     => enPlural nomSg ;
         < x+"e" , Masc >    => nPlural nomSg ;  -- Regel 3
         -- 4.Fall: er/r-Plural 
         < x+"tum", Neut >   => erPlural (x+"tüm") ;
         < x@("Irr" | "Reich")+"tum",
           Masc >            => erPlural (x+"tüm") ;
         -- 5.Fall: sPlural, falls auf klingenden Vokal endend
         < _ + #vowel, 
           _g >              => sPlural nomSg ;

         -- tendenzielle Fälle
         -- 6.Fall: 89%|74%
         < x+"e"+(""|#consonant),
            Masc|Neut >      => nullPlural nomSg ; -- Regel 3
         < _,
            Masc|Neut >      => ePlural nomSg ; 
         -- 7.Fall: 73% 
         < x+"e"+(""|#consonant), 
           Fem >             =>  nPlural nomSg ; -- Regel 3
         < _ , Fem >         => enPlural nomSg ;
         -- 8.Fall 
         < _ , Neut >        => erPlural nomSg ; -- UNERREICHT
         _else               => table { c => "TODO" } 
         } ;

  param 
    PlType = enPl | nPl | ePl | nullPl | erPl | sPl ;
    SgType = enSg | nSg | esSg | sSg | nullSg ;

  oper
    declPl2 : Str -> Gender -> PlType -> (Case => Str) =
     \s,g,t -> case t of {
                           enPl => enPlural s ;
                           nPl  => nPlural s ;
                           ePl  => ePlural s ;
                         nullPl => nullPlural s ;
                           erPl => erPlural s ;
                           _    => sPlural s 
    } ;

    declSg2 : Str -> Gender -> SgType -> (Case => Str) =
     \s,g,t -> case t of {
                           enSg => enSingular s ;
                            nSg =>  nSingular s ;
                           esSg => esSingular s ;
                            sSg =>  sSingular s ;
                              _ => nullSingular s 
    } ;

    enSingular x = table Case [x; x+"en"; x+"en"; x+"en"] ;
     nSingular x = table Case [x; x+"n";  x+"n";  x+"n" ] ;
    esSingular x = table Case [x; x+"es"; x+"e";  x] ;
     sSingular x = table Case [x; x+"s";  x;      x] ;
    nullSingular x = table Case [x; x; x; x] ;

    declPl = overload {
        declPl : Str -> Gender -> (Case => Str) = declPl' ;
        declPl : Str -> Gender -> PlType -> (Case => Str) = declPl2 ;
    } ; 
    declSg = overload {
        declSg : Str -> Gender -> (Case => Str) = declSg' ;
        declSg : Str -> Gender -> SgType -> (Case => Str) = declSg2 ;
    } ; 

    declDefault : Str -> Gender -> N = \nomSg,g -> 
        { s = table { Sg => declSg' nomSg g ;
                      Pl => declPl' nomSg g } ;
          g = g } ;

    mkN = overload {
             mkN : Str -> Gender -> N = declDefault ;   -- nur nomSg
             mkN : Str -> Gender -> SgType -> N = \nomSg,g,ty ->
                   { s = table{ Sg => declSg nomSg g ty ;
                                Pl => declPl nomSg g } ;
                     g = g } ;
             mkN : Str -> Gender -> PlType -> N = \nomSg,g,ty ->
                   { s = table{ Sg => declSg nomSg g ;
                                Pl => declPl nomSg g ty } ;
                     g = g } ;
             mkN : Str -> Gender -> SgType -> PlType -> N = \nomSg,g,tySg,tyPl ->
                   { s = table{ Sg => declSg nomSg g tySg;
                                Pl => declPl nomSg g tyPl } ;
                     g = g } ;
             mkN : Str -> Str -> Gender -> N = \nomSg,nomPl,g -> 
                   { s = table{ Sg => declSg nomSg g ;
                                Pl => declPl nomPl g } ;
                     g = g } ;
             mkN : Str -> Str -> Gender -> SgType -> N = \nomSg,nomPl,g,ty -> 
                   { s = table{ Sg => declSg nomSg g ty ;
                                Pl => declPl nomPl g } ;
                     g = g } ;
             mkN : Str -> Str -> Gender -> PlType -> N = \nomSg,nomPl,g,ty -> 
                   { s = table{ Sg => declSg nomSg g ;
                                Pl => declPl nomPl g ty } ;
                     g = g } ;
             mkN : Str -> Str -> Gender -> SgType -> PlType -> N = 
                   \nomSg,nomPl,g,tySg,tyPl -> 
                   { s = table{ Sg => declSg nomSg g tySg;
                                Pl => declPl nomPl g tyPl } ;
                     g = g } ;
    } ;
}
